#include "tagremover_alg.h"
#include <iostream>
#include <sstream>
#include <string>
#include <algorithm>
#include <map>
#include <exception>
#include <iterator>

using std::string;

struct TagPredicate {
    bool operator()(char c);
private:
    bool in_tag{false};
};

bool TagPredicate::operator()(char c)
{
    bool res = in_tag;
    if(c == '<') res = in_tag = true;
    else if (in_tag && (c == '>')) in_tag=false;

    return !res;
}

std::ostream& remove_tags(std::istream& in, std::ostream& out)
{
    auto iit = std::istream_iterator<char>(in);
    auto oit = std::ostream_iterator<char>(out);
    std::copy_if(iit, std::istream_iterator<char){}, oit, TagPredicate());
    return out;
}

std::map<string,string> symbols{ {"&lt;","<"}, {"&gt;",">"},{"&nbsp;"," "}, {"&amp;","&"}};

template <typename T>
struct TD;

template <typename T>
struct SymbolFilter :std::iterator< std::output_iterator_tag, char> {

    struct FilterBuffer:std::iterator< std::output_iterator_tag, char> {
        FilterBuffer(SymbolFilter* f) :filter(f) {}
        SymbolFilter& operator=(char c)
        {
            auto in_sym = filtering;
            if(filtering || (c == '&')){
                in_sym = filtering = true;
                buffer.push_back(c);
            } 
            if (filtering && (c == ';')){
                filtering = false;
                auto s = symbols.find(buffer);
                if(s != symbols.end()){
                    for(auto x : s->second) *(filter->out++)= x;
                    buffer.clear();
                }
            }
            if(!in_sym) {
                *filter->out++ = c;
            } else {
            }
            // return out;
            return *filter;
        }
    private:
        std::string buffer;
        bool filtering{false};
        SymbolFilter* filter;
    };

    SymbolFilter(T t) :out(t),buffer(this) {};

    FilterBuffer& operator*() {return buffer;}
    T operator++() {return ++out;}
    // operator T () {return out;}
private:
    T out;
    FilterBuffer buffer;
};

// template <typename T>
// T& SymbolFilter<T>::FilterBuffer::operator=(char c)
// template <typename T>
// typename SymbolFilter<T>::FilterBuffer& SymbolFilter<T>::operator*()
// {
//     return buffer;
// }
void TagRemover::replace_symbols()
{
    // std::string res;
    // auto r =std::back_inserter(res);
    auto e = std::copy(buffer.begin(), buffer.end(), SymbolFilter<decltype(buffer.begin())>(buffer.begin()));
    // auto e = std::copy(buffer.begin(), buffer.end(), SymbolFilter<decltype(r)>(r));
    // buffer.erase(e, buffer.end());
    //
    // std::cout << "replaced symbols:\n" << res << std::endl;
}



TagRemover setup()
{
    std::string raw{"<html><Head>Head</Head><body><h1>Hej!</h1><p>Testing, testing. a &lt; b b &gt; c</p></body></html>"};
    return TagRemover(raw);
}


void test_remove_tags()
{
    std::cout << "test_remove_tags:\n";
    std::string expected{"HeadHej!Testing, testing. a &lt; b b &gt; c"};

    auto tr = setup();
    tr.remove_tags();

    std::string actual = tr.get_buffer();
    if(actual != expected) {
        std::cout << "ERROR: expected: "<< expected << "\n" <<
                     "       got     : "<< actual << std::endl;
    } else {
        std::cout << "OK.\n";
    }

}
void test_replace_symbols()
{
    std::cout << "test_replace_symbols:\n";

    std::string raw{"<html><Head>Head</Head><body><h1>Hej!</h1><p>Testing, testing. a &lt; b</p></body></html>"};
    std::string expected{"HeadHej!Testing, testing. a < b b > c"};
    auto tr = setup();
    tr.remove_tags();
    tr.replace_symbols();

    std::string actual = tr.get_buffer();
    if(actual != expected) {
        std::cout << "ERROR: expected: "<< expected << "\n" <<
                     "       got     : "<< actual << std::endl;
    } else {
        std::cout << "OK.\n";
    }

}

void test_string_ops()
{
    string s = "Test <TAG> testing <tag>";

    auto tb = s.find("<");
    std::cout << "tb: " << tb << std::endl;
    auto te = s.find(">", tb);
    std::cout << "te: " << te << std::endl;

    s.erase(tb,te-tb+1);
    std::cout << "s: " << s << std::endl;
}
void test_algorithm()
{
    string s = "Test <TAG> testing <tag>";

    auto tb = std::find(s.begin(), s.end(), '<');
    std::cout << "tb: " << *tb << std::endl;
    auto te = std::find(tb, s.end(), '>');
    std::cout << "te: " << *te << std::endl;

    s.erase(tb,++te);
    std::cout << "s: " << s << std::endl;
}

void TagRemover::print(std::ostream& os)
{
    remove_tags();
    replace_symbols();
    os << buffer;
}

void translate(std::istream& is, std::ostream& os)
{
    TagRemover tr(is);
    tr.print(os);
}

int main()
{
#ifdef TEST
    // test_string_ops();
    // test_algorithm();
    test_remove_tags();
    test_replace_symbols();
#else
    translate(std::cin, std::cout);
#endif
}
